/*
  Modified Reliability 2 trials.sas, by adding an extra trial.
  The analysis is simple test-retest reliability, allowing for extra error on Trial 1 
(e.g., due to familiarization). The %let values also allow for extra error in Trials 2
and 3 (e.g., due to individual responses to an intervention), but these equal 0 here.
See the next program (06 Time series.sas) for non-zero values in Trials 2 and 3.
  Modify the program as shown in the proc mixed step use the repeated statement.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=VO2max; *the dependent variable;
%let mean=380; *true grand mean of dependent, back transforms to exp(380/100)=44.7;
%let sd=10; *true between-subject SD;
%let tem=3; *typical error;
%let deltamean12=2; *change in mean Trial2-Trial1;
%let deltamean13=3; *change in mean Trial3-Trial1;
%let extraerror1=2; *more error on Trial1;
%let extraerror2=0; *more error on Trial2;
%let extraerror3=0; *more error on Trial3;
%let extraerror23=0; *more shared error on Trial2 and Trial3;
%let ss=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do ChildID=1 to &ss;
  True&dep=&mean+&sd*rannor(0);
  XtraError23=&extraerror23*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltamean12+&tem*rannor(0)+&extraerror2*rannor(0)+XtraError23;
  output;
  Trial=3;
  Obsvd&dep=True&dep+&deltamean13+&tem*rannor(0)+&extraerror3*rannor(0)+XtraError23;
  output;
  end;

title "The raw data, the first 6 obs";
proc print data=dat1(obs=6);
format True&dep Obsvd&dep 5.1;
run;

title "Simple stats";
proc means maxdec=1;
var True&dep Obsvd&dep;
class Trial;
run;

*generate means and SDs for plotting;
proc sort data=dat1;
by Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1 means1;

ods graphics / reset width=10cm height=13cm imagemap attrpriority=none;
title "Line diagram of individual values, and means and SDs";
proc sgplot data=datplot noautolegend;
*styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
series x=Trial y=Obsvd&dep/lineattrs=(pattern=solid color=lightgrey) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black);
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markerfillattrs=(color=white) 
  markeroutlineattrs=(color=black) markerattrs=(size=12 symbol=circlefilled);
xaxis values=(1,2,3) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
run;

data dat2;
set dat1;
XvarTrial1=0; *dummy variable to estimate extra variance on Trial 1;
if Trial=1 then XvarTrial1=1;
XvarTrial2=0; *dummy variable to estimate extra variance on Trial 2;
if Trial=2 then XvarTrial2=1;
XvarTrial3=0; *dummy variable to estimate extra variance on Trial 3;
if Trial=3 then XvarTrial3=1;
XvarTrial23=0; *dummy variable to estimate extra variance shared by Trials 2 and 3;
if Trial=2 or Trial=3 then XvarTrial23=1;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Reliability mixed model; Sample size=&ss; True mean=&mean; True between SD=&sd; Delta mean12,13=&deltamean12,&deltamean13";
title2 "Typical error=&tem; Extra error Trial1,2,3=&extraerror1,&extraerror2,&extraerror3; Extra error Trial23=&extraerror23";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Trial;
model Obsvd&dep=Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
*Simple rely model with extra variance on Trial 1:;
random Int XvarTrial1/subject=ChildID s cl alpha=&alpha;
*Star off the above and unstar the repeated below for the most general random-effects
 structure, but no error estimates.;
*repeated Trial/subject=ChildID type=un; 
estimate "Trial 1 mean" Trial 1 0 0/cl alpha=&alpha;
estimate "Trial 2 mean" Trial 0 1 0/cl alpha=&alpha;
estimate "Trial 2-1" Trial -1 1 0/cl alpha=&alpha;
estimate "Trial 3-1" Trial -1 0 1/cl alpha=&alpha;
lsmeans Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms (random effects)";
title4 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1;
length Group $ 10; *in case there is a group= in the random effects;
set cov;
if StdErr ne .;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&decdep CLtd 5.2 DegFree 5.1; 
run;

title3 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title3 "Least-squares means";
proc print data=lsm1;
var Effect Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title3 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Trial;
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title3 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;








