/*
  This program adds a third trial to 08 Controlled trial 2 trials.sas. It simulates 
extra error on the first trial (e.g., due to familiarization), and extra errors 
on Trials 2 and 3 to represent individual responses to the experimental treatment.
Part of the individual responses are shared between Trials 2 and 3. Part of the
shared individual responses is correlated with the subject's true ability (true 
&dep) via a value for covarfactor, which is the slope of the regression line 
for individual responses vs ability. Sample values for covarfactor are derived via 
the covariance between ability and individual responses, and from the slope of the 
line of individual responses vs ability derived from the random-effect solution.
  Modify the program as shown in the Proc Mixed step to use the repeated statement.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
*c=control, e=exptal;
%let dep=VO2max; *the dependent variable;
%let meanc=380; *true grand mean in the first trial, back transforms to exp(380/100)=44.7;
%let meane=380; *true grand mean exptal in the first trial, same as meanc, if assume kids well randomized to control and exptal;
%let sdc=10; *true between-subject SD;
%let sde=10; *true between-subject SD;
%let tem=3; *typical (standard) error of measurement;
*%let teme=3; *typical (standard) error of measurement, make it the same in both groups;
%let deltameanc2=1; *change in control mean Trial 2 minus Trial 1, e.g., placebo effect;
%let deltameane2=5; *change in exptal mean Trial 2 minus Trial 1;
%let deltameanc3=2; *change in control mean Trial 3 minus Trial 1, e.g., placebo effect;
%let deltameane3=8; *change in exptal mean Trial 3 minus Trial 1;
%let extraerror1=2; *more error on Trial 1 in both groups, due to familiarization;
*%let extraerror1e=0; *more error on Trial 1, due to familiarization;
*%let extraerror2c=0; *extra error on Trial 2, due to individual responses;
*%let extraerror3c=0; *extra error on Trial 3, due to individual responses;
%let extraerror2e=2; *extra error on Trial 2, due to individual responses;
%let extraerror23e=3; *extra error on Trials 2 and 3 due to shared individual responses;
%let extraerror3e=3; *extra error on Trial 3, due to individual responses;
%let covarfactor=-0.3; *fraction or multiple of subject's true SD contributing to 
 correlated individual responses in exptal group;
%let ssizec=20; *sample size;
%let ssizee=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for SDs and effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
Treat="control";
do ChildID=1 to &ssizec;
  True&dep=&meanc+&sdc*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameanc2+&tem*rannor(0);
  output;
  Trial=3;
  Obsvd&dep=True&dep+&deltameanc3+&tem*rannor(0);
  output;
  end;
Treat="exptal";
do ChildID=&ssizec+1 to &ssizec+&ssizee;
  True&dep=&meane+&sde*rannor(0);
  indresp23=&extraerror23e*rannor(0)+&covarfactor*(True&dep-&meane);
  Trial=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameane2+&tem*rannor(0)+&extraerror2e*rannor(0)+indresp23;
  output;
  Trial=3;
  Obsvd&dep=True&dep+&deltameane3+&tem*rannor(0)+&extraerror3e*rannor(0)+indresp23;
  output;
  end;

title "Raw data, first 10 observations (which will miss the exptal)";
proc print data=dat1(obs=10);
format True&dep Obsvd&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var True&dep Obsvd&dep;
class Treat Trial;
run;

*generate means and SDs for plotting;
proc sort data=dat1;
by Treat Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Treat Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1(in=a) means1;
*trick to get colored plots for individual values;
if a then do;
  if Treat="control" then Obsvd&dep.con=Obsvd&dep;
  else Obsvd&dep.exp=Obsvd&dep;
  end;
run;

ods graphics / reset width=10cm height=13cm imagemap attrpriority=none;
title "Line diagram of individual values, and means and SDs";
proc sgplot data=datplot;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
scatter x=Trial y=Mean/filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.15 name="points"; *duplication of points to get legend;
series x=Trial y=Obsvd&dep.con/lineattrs=(pattern=solid color=lightskyblue) group=ChildID;
series x=Trial y=Obsvd&dep.exp/lineattrs=(pattern=solid color=pink) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black)
  group=Treat groupdisplay=cluster clusterwidth=0.15;
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Treat groupdisplay=cluster clusterwidth=0.15;
xaxis values=(1,2,3) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
legenditem type=line name="linec"/lineattrs=(color=lightskyblue) label="control"; 
legenditem type=line name="linee"/lineattrs=(color=pink) label="exptal";
keylegend "points" "linec" "linee";
run;

*Set up dummy variables for estimating extra error in Trial 1 in both groups,
 and individual responses in Trials 2 and 3 in the exptal group;
data dat2;
set dat1;
XvarTrial1=0; *more error on Trial 1;
if Trial=1 then XvarTrial1=1;
XvarExpTrial23=0; *individual responses in exptal group on Trials 2 and 3;
if Treat="exptal" and (Trial=2 or Trial=3) then XvarExpTrial23=1;
XvarExpTrial2=0; *extra individual responses in exptal group on Trial 2;
if Treat="exptal" and Trial=2 then XvarExpTrial2=1;
XvarExpTrial3=0; *extra individual responses in exptal group;
if Treat="exptal" and Trial=3 then XvarExpTrial3=1;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Controlled-trial mixed model. Control, Exptal: Sample size=&ssizec, &ssizee; True mean=&meanc, &meane; True between SD=&sdc, &sde";
title2 "Delta mean Trial2-1=&deltameanc2, &deltameane2; Delta mean Trial3-1=&deltameanc3, &deltameane3; Typical error=&tem; Extra error Trial1=&extraerror1";
title3 "Indiv resp expt Trials 2,23,3=&extraerror2e, &extraerror23e, &extraerror3e; Extra indiv resp Trial23 correlated with Trial1=&covarfactor*&sde";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Treat Trial; *always make the order here the same in the model statemnt;
model Obsvd&dep=Treat*Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random XvarTrial1/subject=ChildID s cl alpha=&alpha;
random Int XvarExpTrial2 XvarExpTrial3/subject=ChildID type=un s cl alpha=&alpha group=Treat;
parms 5  100 0 0  0 0 0  100 5 5  5 5 5  5;
*Star off the above and unstar the repeated below to allow for the most general covparms,
 but without estimation of individual responses. Disregard the analysis of the covparms;
*repeated Trial/subject=ChildID type=un group=Treat;
estimate "Trial 2-1 cont" Treat*Trial -1 1 0  0 0 0/cl alpha=&alpha;
estimate "Trial 2-1 expt" Treat*Trial 0 0 0  -1 1 0/cl alpha=&alpha;
estimate "Trial 2-1 expt-cont" Treat*Trial 1 -1 0  -1 1 0/cl alpha=&alpha;
estimate "";
estimate "Trial 3-1 cont" Treat*Trial -1 0 1  0 0 0/cl alpha=&alpha;
estimate "Trial 3-1 expt" Treat*Trial 0 0 0  -1 0 1/cl alpha=&alpha;
estimate "Trial 3-1 expt-cont" Treat*Trial 1 0 -1  -1 0 1/cl alpha=&alpha;
lsmeans Treat*Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title4 "Class levels";
proc print data=clev;
run;

title4 "Covparms (random effects)";
title5 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1 covboot;
length CovParm $ 12;
set cov;
DegFree=2*ZValue**2; *Degrees of freedom;
retain LagEst LagSE;
Est=Estimate; SE=StdErr;
if covparm="UN(1,1)" then do;
  covparm="ChildID";
  LagEst=Estimate;
  LagSE=StdErr;
  end;
if covparm="UN(2,1)" then do;
  covparm="CovFactor2";
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
if covparm="UN(2,2)" then covparm="IndivResp2";
if covparm="UN(3,1)" then do;
  covparm="CovFactor3";
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
if covparm="UN(3,2)" then covparm="IndivResp23";
if covparm="UN(3,3)" then covparm="IndivResp3";
array a Estimate Lower Upper; 
if index(CovParm,"CovFactor")=0 then do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statemnt and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
if Estimate ne 0 then output cov1;
run;

title4 "Covparm variances as SD; covariances as slopes";
title5 "True indiv resp Trials 2,3 (IndivResp2,IndivResp3)=sqrt(&extraerror2e^2+&extraerror23e^2+[&covarfactor*&sde]^2), sqrt(&extraerror3e^2+&extraerror23e^2+[&covarfactor*&sde]^2)";
title6 "CovFactor is the effect of true ability on individual response as a slope; true values both=&covarfactor";
title7 "True shared indiv resp (IndivResp23)=sqrt(&extraerror23e^2+[&covarfactor*&sde]^2)";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;


*make macro variables for 5th and 95th percentiles;
data _null_;
call symput('pctlow',&alpha*100/2);
call symput('pctupp',100-&alpha*100/2);

proc univariate noprint data=covboot;
var EstimateBoot Estimate;
output out=covboot1(drop=d) mean=d Estimate median=EstimateBoot pctlpre=X pctlpts=&pctlow,&pctupp
 pctlname=Lower Upper;
by Group CovParm;
run;

data covboot2;
set covboot1(rename=(XLower=Lower XUpper=Upper));
CLpmApprox=(Upper-Lower)/2;

data cov1a;
set cov1;
if index(CovParm,"CovFactor");

data cov2;
merge cov1a covboot2;
by Group CovParm;

title5 "CovFactor with an extra decimal place, and bootstrapped estimate and confidence limits";
title6 "CovFactor is the effect of true ability on individual response as a slope; true values both=&covarfactor";
proc print data=cov2;
var CovParm Group Estimate EstimateBoot CLpmApprox Lower Upper Alpha DegFree;
format Estimate EstimateBoot Lower Upper CLpmApprox 5.2 CLtd 5.2 DegFree 5.1; 
run;

title4 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title4 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title4 "Least-squares means";
proc print data=lsm1;
var Effect Treat Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title4 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Treat _Treat Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title4 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Treat;
run;
ods graphics / reset;

title4 "Outliers (Standardized residual >&tvalue)";
title5 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title4 "Random-effects solution, first 10 observations";
title5 "View the whole dataset to see the values of the exptal group";
proc print data=solr(obs=20);
format Estimate StdErrPred Lower Upper 5.&deceff;
where Estimate ne 0; *suppress the irrelevant non-contributing values;
run;

*plot individual responses vs ability;
data abil;
set solr;
if Effect="Intercept";
if Estimate ne 0;
keep ChildID Estimate;

data indresp;
set solr;
if index(Effect,"XvarExp");
if Estimate ne 0;
keep ChildID Effect Estimate;

data abilindresp;
merge abil(rename=(Estimate=Ability)) indresp(rename=(Estimate=IndivResp));
by ChildID;
if Effect ne ""; *delete control subjects;

title4 "Relative ability and individual responses from solution for random effects";
title5 "First 10 observations";
proc print data=abilindresp(obs=10);
format Ability IndivResp 5.&deceff;
run;

proc sort;
by Effect;

ods graphics / reset width=13cm height=13cm imagemap attrpriority=none;
title4 "Scatterplot of individual responses vs relative ability";
proc sgplot data=abilindresp;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
refline 0/axis=x lineattrs=(pattern=dot thickness=1 color=black);
reg x=Ability y=IndivResp/nomarkers lineattrs=(thickness=1) group=Effect name="lines";
scatter x=ability y=IndivResp/filledoutlinedmarkers markerattrs=(size=12) group=Effect name="points";
yaxis label="Relative individual response" labelpos=top;
xaxis label="Relative ability";
keylegend "points" "lines";
run;

data est;

ods select none;
title4 "Slope of individual responses vs relative ability";
title5 "True slopes both=&covarfactor; Uncertainty provided by the regression is not shown, because too unrealistically precise";
proc mixed data=abilindresp covtest cl alpha=&alpha nobound;
model IndivResp=Ability/ddfm=sat solution cl residual alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Slope" Ability 1/cl alpha=&alpha;
ods output estimates=est;
by Effect;
run;
ods select all;

*data step to stop getting an error when est is empty;
data est1;
set est;
keep Effect Label Estimate;
format Estimate 5.2;

proc print data=est1;
run;

