/*
  Modified 05 Reliability 3 trials.sas, by making two testing occasions (Week1 and 
Week2). To simulate a reliability study, extra random error is added to Trial 1 due
to familiarization on both occasions. Extra error can also be added to Trials 2 and
3 to simulate an intervention with individual responses following Trial 1, but these
are set to 0 here. 
  Modify the program as shown in the Proc Mixed step to use the repeated statement.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=VO2max; *the dependent variable;
%let mean=380; *true grand mean of dependent, back transforms to exp(380/100)=44.7;
%let sd=10; *true between-subject SD;
%let tem=3; *typical error;

%let deltamean12=2; *change in mean Trial2-Trial1;
%let deltamean13=2; *change in mean Trial3-Trial1;
%let extraerror1=2; *extra error on Trial1, random in both occasions;
%let extraerror2=0; *extra error on Trial2, consistent in both occasions;
%let extraerror3=0; *extra error on Trial3, consistent in both occasions;
%let extraerror23=0; *extra shared error on Trial2 and Trial3, consistent in both occasions;

%let deltameanoccasion=1; *change in mean Occasion 2-Occasion1;
%let extraerroroccasion=2; *within-subject variability between occasions;

%let ss=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do ChildID=1 to &ss;
  True&dep=&mean+&sd*rannor(0);
  XtraError2=&extraerror2*rannor(0);
  XtraError3=&extraerror3*rannor(0);
  XtraError23=&extraerror23*rannor(0);
  
  Occasion="Week1";
  True&dep.1=True&dep+&extraerroroccasion*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep.1+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep.1+&deltamean12+&tem*rannor(0)+XtraError2+XtraError23;
  output;
  Trial=3;
  Obsvd&dep=True&dep.1+&deltamean13+&tem*rannor(0)+XtraError3+XtraError23;
  output;
  
  Occasion="Week2";
  True&dep.2=True&dep+&deltameanoccasion+&extraerroroccasion*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep.2+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep.2+&deltamean12+&tem*rannor(0)+XtraError2+XtraError23;
  output;
  Trial=3;
  Obsvd&dep=True&dep.2+&deltamean13+&tem*rannor(0)+XtraError3+XtraError23;
  output;
  end;

title "The raw data, the first 12 obs";
proc print data=dat1(obs=12);
*format True&dep True&dep.1 True&dep.2 Obsvd&dep 5.1;
run;

title "Simple stats";
proc means maxdec=1;
var True&dep True&dep.1 True&dep.2  Obsvd&dep;
class Occasion Trial;
run;

*create variable Week to plot trials and weeks;
data dat1a;
set dat1;
Week=(Trial-2)/10+substr(Occasion,5,1);

proc sort;
by Week;

proc means noprint data=dat1a;
var Obsvd&dep;
by Week;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1a means1;
run;

ods graphics / reset width=8cm height=11cm imagemap attrpriority=none;
title "Line diagram of individual values and means";
proc sgplot data=datplot noautolegend;
*styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
series x=Week y=Obsvd&dep/lineattrs=(pattern=solid color=lightgrey) group=ChildID;
series x=Week y=Mean/lineattrs=(pattern=solid color=black);
scatter x=Week y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markerfillattrs=(color=white) 
  markeroutlineattrs=(color=black) markerattrs=(size=12 symbol=circlefilled);
xaxis values=(1,2) offsetmin=0.15 offsetmax=0.15;
yaxis label="&dep" labelpos=top;
run;

data dat2;
set dat1a;
XvarTrial1=0; *dummy variable to estimate extra variance on Trial 2;
if Trial=1 then XvarTrial1=1;
XvarTrial2=0; *dummy variable to estimate extra variance on Trial 2;
if Trial=2 then XvarTrial2=1;
XvarTrial3=0; *dummy variable to estimate extra variance on Trial 3;
if Trial=3 then XvarTrial3=1;
XvarTrial23=0; *dummy variable to estimate extra variance shared by Trials 2 and 3;
if Trial=2 or Trial=3 then XvarTrial23=1;

*proc print;run;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Rely mixed model; Sample size=&ss; True mean=&mean; True between SD=&sd; Delta mean Trial12,13=&deltamean12,&deltamean13";
title2 "Typical error=&tem; Extra residual error Trial1=&extraerror1; Extra error Trial2,3=&extraerror2,&extraerror3; Extra error Trial23=&extraerror23";
title3 "Delta mean Occasion12=&deltameanoccasion; Occasion error=&extraerroroccasion";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Occasion Trial; *always make the order here the same in the model statement;
model Obsvd&dep=Occasion Trial Occasion*Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
*reliability random effects:;
random Int XvarTrial1*Occasion Occasion/subject=ChildID s cl alpha=&alpha;
parms 100 1 1 10;
*Star off the above and unstar the repeated below to allow for the most general covparms,
 but without estimation of errors. Disregard the analysis of the covparms.;
*repeated Occasion*Trial/subject=ChildID type=un; 
estimate "Trial 1 mean" Trial 1 0 0  Occasion .5 .5  Occasion*Trial .5 0 0  .5 0 0/cl alpha=&alpha;
estimate "Trial 2 mean" Trial 0 1 0  Occasion .5 .5  Occasion*Trial 0 .5 0  0 .5 0/cl alpha=&alpha;
estimate "Trial 3 mean" Trial 0 0 1  Occasion .5 .5  Occasion*Trial 0 0 .5  0 0 .5/cl alpha=&alpha;
estimate "Trial 2-1 " Trial -1 1 0  Occasion*Trial -.5 .5 0  -.5 .5 0/cl alpha=&alpha;
estimate "Trial 3-1 " Trial -1 0 1  Occasion*Trial -.5 0 .5  -.5 0 .5/cl alpha=&alpha;
*estimate "Trial 2-1" Trial -1 1 0/cl alpha=&alpha; *this works without Occasion*Trial, not clear why;
estimate "";
estimate "Week 2-1" Occasion -1 1/cl alpha=&alpha; *this also works without Occasion*Trial;
lsmeans Occasion Trial Occasion*Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title4 "Class levels";
proc print data=clev;
run;

title4 "Covparms (random effects)";
title5 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1;
length Group $ 10; **in case there is a group= in the random effects;
set cov;
if StdErr ne .;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title4 "Covparms (random effects) as SD";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&decdep CLtd 5.2 DegFree 5.1; 
run;

title4 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title4 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title4 "Least-squares means";
proc print data=lsm1;
var Effect Occasion Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title4 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Occasion _Occasion Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title4 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Week;
run;
ods graphics / reset;

title4 "Outliers (Standardized residual >&tvalue)";
title5 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title4 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;








